"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const graphql_codegen_visitor_plugin_common_1 = require("graphql-codegen-visitor-plugin-common");
const graphql_1 = require("graphql");
class FlowSelectionSetToObject extends graphql_codegen_visitor_plugin_common_1.SelectionSetToObject {
    constructor(_scalars, _schema, _convertName, _addTypename, _visitorConfig, _parentSchemaType, _selectionSet) {
        super(_scalars, _schema, _convertName, _addTypename, _parentSchemaType, _selectionSet);
        this._visitorConfig = _visitorConfig;
    }
    createNext(parentSchemaType, selectionSet) {
        return new FlowSelectionSetToObject(this._scalars, this._schema, this._convertName, this._addTypename, this._visitorConfig, parentSchemaType, selectionSet);
    }
    buildPrimitiveFields(parentName, fields) {
        if (fields.length === 0) {
            return null;
        }
        const useFlowExactObject = this._visitorConfig.useFlowExactObjects;
        const useFlowReadOnlyTypes = this._visitorConfig.useFlowReadOnlyTypes;
        return `$Pick<${parentName}, {${useFlowExactObject ? '|' : ''} ${fields
            .map(fieldName => `${useFlowReadOnlyTypes ? '+' : ''}${fieldName}: *`)
            .join(', ')} ${useFlowExactObject ? '|' : ''}}>`;
    }
    buildLinkFields(fields) {
        if (fields.length === 0) {
            return null;
        }
        const useFlowExactObject = this._visitorConfig.useFlowExactObjects;
        const useFlowReadOnlyTypes = this._visitorConfig.useFlowReadOnlyTypes;
        return `{${useFlowExactObject ? '|' : ''} ${fields
            .map(field => `${useFlowReadOnlyTypes ? '+' : ''}${field.alias || field.name}: ${field.selectionSet}`)
            .join(', ')} ${useFlowExactObject ? '|' : ''}}`;
    }
    buildAliasedPrimitiveFields(parentName, fields) {
        if (fields.length === 0) {
            return null;
        }
        const useFlowExactObject = this._visitorConfig.useFlowExactObjects;
        const useFlowReadOnlyTypes = this._visitorConfig.useFlowReadOnlyTypes;
        return `{${useFlowExactObject ? '|' : ''} ${fields
            .map(aliasedField => `${useFlowReadOnlyTypes ? '+' : ''}${aliasedField.alias}: $ElementType<${parentName}, '${aliasedField.fieldName}'>`)
            .join(', ')} ${useFlowExactObject ? '|' : ''}}`;
    }
    clearOptional(str) {
        if (str.startsWith('?')) {
            return str.substring(1);
        }
        return str;
    }
    wrapTypeWithModifiers(baseType, type) {
        if (graphql_1.isNonNullType(type)) {
            return this.clearOptional(this.wrapTypeWithModifiers(baseType, type.ofType));
        }
        else if (graphql_1.isListType(type)) {
            const innerType = this.wrapTypeWithModifiers(baseType, type.ofType);
            return `?Array<${innerType}>`;
        }
        else {
            return `?${baseType}`;
        }
    }
}
exports.FlowSelectionSetToObject = FlowSelectionSetToObject;
//# sourceMappingURL=flow-selection-set-to-object.js.map